User attempted the step multiple times but all {{ max_retries }} retries failed. Summarize the main reason why the actions failed based on the provided screenshot, page HTML, user goal and details.

Make sure to ONLY return the JSON object in this format with no additional text before or after it:
```json
{
  "page_info": str, // Think step by step. Describe useful information from the page HTML related to the user goal.
  "reasoning": str, // Think step by step. Summarize why the actions failed based on 'page_info', screenshots, user goal and the failed actions. Keep it short and to the point.
}
```

User Goal:
{{ navigation_goal }}

User Details:
{{ navigation_payload }}

Failed Actions In Each Retry Step:
{% for step in steps %}Retry Step {{ step.order }} -- {{ step.actions_result }}
{% endfor %}

Page HTML:
```html
{{ page_html }}
```

Current datetime, ISO format:
```
{{ local_datetime }}
```
