Given a browser action, come up with a one-line prompt suitable for a browser agent, a block label, and a title.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in JSON format with the following keys:
{
  "prompt": str, // A one-line prompt suitable for a browser agent describing the user action.
  "title": str, // A descriptive and informative title for the goal. Use no more than 5 words
  "block_label": str // A label for the block. Lower case. Based off of the "title". Underscores are permitted.
}

User action:
```
{{ action }}
```
