"""add vnc support in debug session model

Revision ID: d15d85a702a4
Revises: bbcc08ba09f5
Create Date: 2025-09-12 13:13:37.428674+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d15d85a702a4"
down_revision: Union[str, None] = "bbcc08ba09f5"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "debug_sessions",
        sa.Column("vnc_streaming_supported", sa.Boolean(), server_default=sa.text("true"), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("debug_sessions", "vnc_streaming_supported")
    # ### end Alembic commands ###
