# This file was auto-generated by Fern from our API Definition.

import typing

ActionType = typing.Union[
    typing.Literal[
        "click",
        "input_text",
        "upload_file",
        "download_file",
        "select_option",
        "checkbox",
        "wait",
        "hover",
        "null_action",
        "solve_captcha",
        "terminate",
        "complete",
        "reload_page",
        "close_page",
        "extract",
        "verification_code",
        "goto_url",
        "scroll",
        "keypress",
        "move",
        "drag",
        "left_mouse",
    ],
    typing.Any,
]
