"""add debug_session_id to workflow_runs table

Revision ID: c50ee6f26432
Revises: ac9d96ea0501
Create Date: 2025-10-01 11:16:03.680233+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "c50ee6f26432"
down_revision: Union[str, None] = "ac9d96ea0501"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("debug_session_id", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "debug_session_id")
    # ### end Alembic commands ###
