"""add_webhook_failure_reason

Revision ID: 1d0a10ae2a13
Revises: 044b4a3c3dbc
Create Date: 2025-07-28 16:06:57.319749+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1d0a10ae2a13"
down_revision: Union[str, None] = "044b4a3c3dbc"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("webhook_failure_reason", sa.String(), nullable=True))
    op.add_column("tasks", sa.Column("webhook_failure_reason", sa.String(), nullable=True))
    op.add_column("workflow_runs", sa.Column("webhook_failure_reason", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "webhook_failure_reason")
    op.drop_column("tasks", "webhook_failure_reason")
    op.drop_column("observer_cruises", "webhook_failure_reason")
    # ### end Alembic commands ###
