# This file was auto-generated by Fern from our API Definition.

import typing

RunEngine = typing.Union[
    typing.Literal["skyvern-1.0", "skyvern-2.0", "openai-cua", "anthropic-cua", "ui-tars"], typing.Any
]
