"""add run_with to workflows table and observer_cruises table

Revision ID: ac9d96ea0501
Revises: e0ccabc005af
Create Date: 2025-09-29 19:00:39.108889+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "ac9d96ea0501"
down_revision: Union[str, None] = "e0ccabc005af"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("run_with", sa.String(), nullable=True))
    op.add_column("workflows", sa.Column("run_with", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "run_with")
    op.drop_column("observer_cruises", "run_with")
    # ### end Alembic commands ###
