Evaluate if the following condition(s) are TRUE or FALSE.
{% if conditions|length == 1 %}
Condition: {{ conditions[0] }}
{% else %}
{% for condition in conditions %}
Condition {{ loop.index }}: {{ condition }}
{% endfor %}
{% endif %}
{% if context_json %}

Use this context to understand variable values:
{{ context_json }}
{% endif %}

For each condition, provide IN THIS ORDER:
1. "reasoning": Explain your reasoning for evaluating the condition. Compare the actual values.
2. "result": The boolean result (true/false) based on the condition.
