"""add organization_id to script_blocks

Revision ID: a027553be970
Revises: 92d665af080e
Create Date: 2025-08-09 01:05:07.564940+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "a027553be970"
down_revision: Union[str, None] = "92d665af080e"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("script_blocks", sa.Column("organization_id", sa.String(), nullable=False))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("script_blocks", "organization_id")
    # ### end Alembic commands ###
