"""add workflow sequential setting

Revision ID: 8f208daee7d1
Revises: 67add341e926
Create Date: 2025-09-18 05:29:42.472400+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "8f208daee7d1"
down_revision: Union[str, None] = "67add341e926"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("job_id", sa.String(), nullable=True))
    op.add_column("workflows", sa.Column("run_sequentially", sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "run_sequentially")
    op.drop_column("workflow_runs", "job_id")
    # ### end Alembic commands ###
