"""add input_or_select_context column to the actions table

Revision ID: bbcc08ba09f5
Revises: 8de03b8cb83a
Create Date: 2025-09-11 20:38:47.648188+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "bbcc08ba09f5"
down_revision: Union[str, None] = "8de03b8cb83a"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("actions", sa.Column("input_or_select_context", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("actions", "input_or_select_context")
    # ### end Alembic commands ###
