"""add run_with to workflow_runs table

Revision ID: 6692fb5a3e91
Revises: 4937dff73027
Create Date: 2025-09-19 14:14:56.090820+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "6692fb5a3e91"
down_revision: Union[str, None] = "4937dff73027"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("run_with", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "run_with")
    # ### end Alembic commands ###
