# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CreditCardCredentialResponse(UniversalBaseModel):
    """
    Response model for credit card credentials — non-sensitive fields only.

    SECURITY: Must NEVER include full card number, CVV, expiration date, or card holder name.
    """

    last_four: str = pydantic.Field()
    """
    Last four digits of the credit card number
    """

    brand: str = pydantic.Field()
    """
    Brand of the credit card
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
