"""add_more_info_to_browser_session

Revision ID: 2e997076a3aa
Revises: f63892bfc429
Create Date: 2025-07-17 06:12:04.974095+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "2e997076a3aa"
down_revision: Union[str, None] = "f63892bfc429"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("persistent_browser_sessions", sa.Column("ip_address", sa.String(), nullable=True))
    op.add_column("persistent_browser_sessions", sa.Column("ecs_task_arn", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("persistent_browser_sessions", "ecs_task_arn")
    op.drop_column("persistent_browser_sessions", "ip_address")
    # ### end Alembic commands ###
