Generating a user-facing description for an browser action to help users understand what the action is doing and why.

Action Information:
- Action Type: {{ action_type }}
{% if intention %}- Intention: {{ intention }}{% endif %}
{% if text %}- Text/Value: {{ text }}{% endif %}
{% if select_option %}- Selected Option: {{ select_option }}{% endif %}
{% if file_url %}- File URL: {{ file_url }}{% endif %}
{% if data_extraction_goal %}- Data Extraction Goal: {{ data_extraction_goal }}{% endif %}
{% if data_extraction_schema %}- Data Extraction Schema: {{ data_extraction_schema }}{% endif %}

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Respond with the following JSON format:
```
{
    "reasoning": str // A clear, user-friendly explanation (one sentence, 20 words max) of what this action is doing. Use present tense like "Clicking the submit button" or "Entering the email address". Focus on the action and its purpose.
}
```

