"""add created_by to actions table

Revision ID: f148f36edc09
Revises: afdfd14ac46d
Create Date: 2025-08-22 13:43:07.370605+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "f148f36edc09"
down_revision: Union[str, None] = "afdfd14ac46d"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("actions", sa.Column("created_by", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("actions", "created_by")
    # ### end Alembic commands ###
