Given a wait action, come up with a block label. It should include the duration in seconds, and a short version of a url, if the url exists.

If the duration for the wait action is in milliseconds, so be sure to convert to seconds.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in JSON format with the following keys:
{
  "block_label": str // A label for the block. Lower case. Underscores are permitted.
}

Wait action:
```
{{ action }}
```
