Given a "go to URL" action, come up with a block label suitable for a browser agent. It should include a short version of a url, if the url exists.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in JSON format with the following keys:
{
  "block_label": str // A label for the block. Lower case. Underscores are permitted.
}

go to URL action:
```
{{ action }}
```
