# This file was auto-generated by Fern from our API Definition.

import typing

ContextParameterValue = typing.Union[
    str, int, float, bool, typing.Dict[str, typing.Optional[typing.Any]], typing.List[typing.Optional[typing.Any]]
]
