# This file was auto-generated by Fern from our API Definition.

import typing

from .geo_target import GeoTarget
from .proxy_location import ProxyLocation

WorkflowRunProxyLocation = typing.Union[ProxyLocation, GeoTarget, typing.Dict[str, typing.Optional[typing.Any]]]
