"""Add model column to workflows table

Revision ID: af49ca791fc7
Revises: ecdba16ef45c
Create Date: 2025-05-29 14:35:38.232668+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "af49ca791fc7"
down_revision: Union[str, None] = "ecdba16ef45c"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflows", sa.Column("model", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "model")
    # ### end Alembic commands ###
