"""add script_blocks.input_fields

Revision ID: 7ab8e817802a
Revises: cf6ae2f5013c
Create Date: 2025-12-10 23:56:33.299511+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "7ab8e817802a"
down_revision: Union[str, None] = "cf6ae2f5013c"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("script_blocks", sa.Column("input_fields", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("script_blocks", "input_fields")
    # ### end Alembic commands ###
