Given the user goal, come up with a title for it, and a block label.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in JSON format with the following keys:
{
  "title": str, // A descriptive and informative title for the goal. Use no more than 5 words
  "block_label": str, // A label for the block. Use 1 word only. Based off of the "title".
}

User goal:
```
{{ user_goal }}
```
