# This file was auto-generated by Fern from our API Definition.

import typing

BlockType = typing.Union[
    typing.Literal[
        "task",
        "task_v2",
        "for_loop",
        "conditional",
        "code",
        "text_prompt",
        "download_to_s3",
        "upload_to_s3",
        "file_upload",
        "send_email",
        "file_url_parser",
        "validation",
        "action",
        "navigation",
        "extraction",
        "login",
        "wait",
        "file_download",
        "goto_url",
        "pdf_parser",
        "http_request",
        "human_interaction",
        "print_page",
    ],
    typing.Any,
]
