"""Browser profile testing & login-free workflow support

Revision ID: dc37d888db44
Revises: a1b2c3d4e5f6
Create Date: 2026-02-19 23:17:34.500314+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "dc37d888db44"
down_revision: Union[str, None] = "a1b2c3d4e5f6"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("credentials", sa.Column("browser_profile_id", sa.String(), nullable=True))
    op.add_column("credentials", sa.Column("tested_url", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("credentials", "tested_url")
    op.drop_column("credentials", "browser_profile_id")
    # ### end Alembic commands ###
