"""add secrets col

Revision ID: 1faa2a5869cd
Revises: 135afee6e7bc
Create Date: 2025-12-09 18:56:25.666572+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1faa2a5869cd"
down_revision: Union[str, None] = "135afee6e7bc"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("credentials", sa.Column("secret_label", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("credentials", "secret_label")
    # ### end Alembic commands ###
