"""update migration script

Revision ID: 135afee6e7bc
Revises: 152354699b93
Create Date: 2025-12-07 20:27:07.352740+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "135afee6e7bc"
down_revision: Union[str, None] = "152354699b93"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_run_blocks", sa.Column("executed_branch_id", sa.String(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("executed_branch_expression", sa.String(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("executed_branch_result", sa.Boolean(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("executed_branch_next_block", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_run_blocks", "executed_branch_next_block")
    op.drop_column("workflow_run_blocks", "executed_branch_result")
    op.drop_column("workflow_run_blocks", "executed_branch_expression")
    op.drop_column("workflow_run_blocks", "executed_branch_id")
    # ### end Alembic commands ###
