"""add generate_script to task v2 table

Revision ID: 8de03b8cb83a
Revises: ee2f523ea454
Create Date: 2025-08-31 06:18:37.032634+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "8de03b8cb83a"
down_revision: Union[str, None] = "ee2f523ea454"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "observer_cruises", sa.Column("generate_script", sa.Boolean(), nullable=False, server_default=sa.false())
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("observer_cruises", "generate_script")
    # ### end Alembic commands ###
