"""add fields to workflow_run_blocks table

Revision ID: d135e472622c
Revises: b80c42316c94
Create Date: 2025-10-27 18:43:20.933288+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d135e472622c"
down_revision: Union[str, None] = "b80c42316c94"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_run_blocks", sa.Column("instructions", sa.String(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("positive_descriptor", sa.String(), nullable=True))
    op.add_column("workflow_run_blocks", sa.Column("negative_descriptor", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_run_blocks", "negative_descriptor")
    op.drop_column("workflow_run_blocks", "positive_descriptor")
    op.drop_column("workflow_run_blocks", "instructions")
    # ### end Alembic commands ###
