Your goal is to check whether the format of the date matches the required format 'YYYY-MM-DD' based on the user's goals, user details.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in JSON format with the following keys:
{
"page_info": str, // Think step by step. Describe all the useful information in the page related to the user goal.
"thought": str, // Think step by step. Describe your thought about how you come up the result. Use information you see on the site to explain.
"is_current_format_correct": bool, // True if the current date format is matching the required format 'YYYY-MM-DD'.
"recommended_date": str, // If is_current_format_correct is True, return null. Otherwise, return the recommended date with the correct format 'YYYY-MM-DD'.
}

Current value:
```
{{ current_value }}
```

User goal:
```
{{ navigation_goal }}
```

User details:
```
{{ navigation_payload_str }}
```

Current datetime, ISO format:
```
{{ local_datetime }}
```