# This file was auto-generated by Fern from our API Definition.

import typing

SkyvernForgeSdkSchemasCredentialsCredentialType = typing.Union[
    typing.Literal["password", "credit_card", "secret"], typing.Any
]
