You receive either an email or a text message containing an OTP(like TOTP, Magic Link) to verify the login. Your job is to parse the content, identify the OTP type and value. The value must be from the content.
{% if enforced_otp_type %}
IMPORTANT: The user has specified they expect a "{{ enforced_otp_type }}" type OTP. You MUST extract the {{ enforced_otp_type }} value from the content, even if other OTP types are present. Ignore any other OTP types in the content. If no {{ enforced_otp_type }} is found, set otp_value_found to false.
{% else %}
There should be only one OTP type and one OTP value in the content.
{% endif %}

You should follow the rules below to identify the OTP type and value:
- If it's a Magic Link login, the value is usually a link which must be a valid HTTP or HTTPS URL.
- If it's a TOTP login, The most common value is a code which is a series of digits, although sometimes it may contain letters.

MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in the following JSON format:
{
    "reasoning": str, // How you figure out what the OTP type and value is or why the OTP type and value is missing. Be precise here to explain the data source and the context that makes you believe where the correct OTP type and value is
    "otp_type": str, // the type of OTP. It can be "magic_link" or "totp"
    "otp_value_found": bool, // true if the OTP value is found. false if the OTP value is not found
    "otp_value": str, // the OTP value. If you cannot identify any OTP value, do not come up with a OTP value and return null
}

Received Content containing OTP:
```
{{ content }}
```
