"""add_extra_headers

Revision ID: a9fd7a08469c
Revises: 2be3e0ba85ff
Create Date: 2025-06-19 07:40:10.580993+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "a9fd7a08469c"
down_revision: Union[str, None] = "2be3e0ba85ff"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("extra_http_headers", sa.JSON(), nullable=True))
    op.add_column("tasks", sa.Column("extra_http_headers", sa.JSON(), nullable=True))
    op.add_column("workflow_runs", sa.Column("extra_http_headers", sa.JSON(), nullable=True))
    op.add_column("workflows", sa.Column("extra_http_headers", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "extra_http_headers")
    op.drop_column("workflow_runs", "extra_http_headers")
    op.drop_column("tasks", "extra_http_headers")
    op.drop_column("observer_cruises", "extra_http_headers")
    # ### end Alembic commands ###
