"""add extension for browser session

Revision ID: e393f33ec711
Revises: b4738bd17198
Create Date: 2025-12-24 04:56:38.500907+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "e393f33ec711"
down_revision: Union[str, None] = "b4738bd17198"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("persistent_browser_sessions", sa.Column("extensions", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("persistent_browser_sessions", "extensions")
    # ### end Alembic commands ###
