"""add memory_mb to task_runs

Revision ID: 174dcd456325
Revises: 67784da0203e
Create Date: 2025-12-10 19:15:24.067819+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "174dcd456325"
down_revision: Union[str, None] = "67784da0203e"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("task_runs", sa.Column("memory_mb", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task_runs", "memory_mb")
    # ### end Alembic commands ###
