# This file was auto-generated by Fern from our API Definition.

import typing

from .non_empty_credit_card_credential import NonEmptyCreditCardCredential
from .non_empty_password_credential import NonEmptyPasswordCredential
from .secret_credential import SecretCredential

CreateCredentialRequestCredential = typing.Union[
    NonEmptyPasswordCredential, NonEmptyCreditCardCredential, SecretCredential
]
