"""add_browser_session_proxy_schema

Revision ID: 4937dff73027
Revises: 8f208daee7d1
Create Date: 2025-09-18 09:16:54.130840+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "4937dff73027"
down_revision: Union[str, None] = "8f208daee7d1"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("persistent_browser_sessions", sa.Column("proxy_location", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("persistent_browser_sessions", "proxy_location")
    # ### end Alembic commands ###
