"""add_screenshot_scroll_number

Revision ID: 2c6b27e8e961
Revises: 1517a4ba63fa
Create Date: 2025-06-14 06:49:43.628471+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "2c6b27e8e961"
down_revision: Union[str, None] = "1517a4ba63fa"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("max_screenshot_scrolling_times", sa.Integer(), nullable=True))
    op.add_column("tasks", sa.Column("max_screenshot_scrolling_times", sa.Integer(), nullable=True))
    op.add_column("workflow_runs", sa.Column("max_screenshot_scrolling_times", sa.Integer(), nullable=True))
    op.add_column("workflows", sa.Column("max_screenshot_scrolling_times", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "max_screenshot_scrolling_times")
    op.drop_column("workflow_runs", "max_screenshot_scrolling_times")
    op.drop_column("tasks", "max_screenshot_scrolling_times")
    op.drop_column("observer_cruises", "max_screenshot_scrolling_times")
    # ### end Alembic commands ###
