Your are here to help the user to perform an SELECT_OPTION action on the web page. Use the user instruction, the content of the elements parsed from the page, the screenshots of the page, and user details to determine which element to select.
Each actionable element is tagged with an ID. Only take the action on the elements provided in the HTML elements, do not image any new element.
MAKE SURE YOU OUTPUT VALID JSON. No text before or after JSON, no trailing commas, no comments (//), no unnecessary quotes, etc.

Reply in JSON format with the following keys:
{
    "page_info": str, // Think step by step. Describe all the useful information in the page related to the user instruction.
    "thoughts": str, // Think step by step. What information makes you believe which element to select. Use information you see on the site to explain.
    "actions": array // You are supposed to give only one action("SELECT_OPTION") in the action list. Here's the format of the action:
    [{
        "reasoning": str, // The reasoning behind the action. This reasoning must be user information agnostic. Mention why you chose the action type, and why you chose the element id. Keep the reasoning short and to the point.
        "user_detail_query": str, // Think of this value as a Jeopardy question. Ask the user for the details you need for executing this action. IMPORTANT: This question must be user information agnostic and must NOT contain specific user data (like names, IDs, specific values, etc.). Ask what information is needed generically. Ask the question even if the details are disclosed in user instruction or user details. If it's a dropdown, ask for the relevant information. Otherwise, use null. Examples are: "Which invoice should I download?", "Does the user have any pets?". If the action doesn't require any user details, use null.
        "user_detail_answer": str, // The answer to the `user_detail_query`. The source of this answer can be user instruction or user details. This answer CAN contain specific user data.
        "confidence_float": float, // The confidence of the action. Pick a number between 0.0 and 1.0. 0.0 means no confidence, 1.0 means full confidence
        "action_type": str, // It's a string enum: "SELECT_OPTION". "SELECT_OPTION" is an element you'd like to select an option from.
        "id": str, // The id of the element to take action on. The id has to be one from the elements list.
        "option": {  // The option to select.
            "label": str, // the label of the option if any. MAKE SURE YOU USE THIS LABEL TO SELECT THE OPTION. DO NOT PUT ANYTHING OTHER THAN A VALID OPTION LABEL HERE
            "index": int, // the index corresponding to the option index under the select element.
            "value": str // the value of the option. MAKE SURE YOU USE THIS VALUE TO SELECT THE OPTION. DO NOT PUT ANYTHING OTHER THAN A VALID OPTION VALUE HERE
        }{% if parse_select_feature_enabled %},
        "context": { // Extract the following detailed information from the "reasoning", and double-check the information by analysing the HTML elements.
            "thought": str, // A string to describe how you double-check the context information to ensure the accuracy.
            "field": str, // Which field is this action intended to fill out?
            "is_required": bool, // True if this is a required field, otherwise false.
            "is_search_bar": bool, // True if the element to take the action is a search bar, otherwise false.
            "is_location_input": bool, // True if the element is asking user to input where he lives, otherwise false. For example, it is asking for location, or address, or other similar information. Output False if it only requires ZIP code or postal code.
            "is_date_related": bool, // True if the field is related to date input or select, otherwise false.
            "date_format": str, // The format of the date or datetime to be input. For example YYYY-MM-DD, YYYY-MM-DD HH:MM:SS, DD.MM.YYYY, MM/DD/YYYY, etc. If the field is not related to date input or select, this should be null.
        }{% endif %}
    }]
}

User instruction:
```
{{ navigation_goal }}
```

User details:
```
{{ navigation_payload_str }}
```

The URL of the page you're on right now is `{{ current_url }}`.

HTML elements from `{{ current_url }}`:
```
{{ elements }}
```
{% if action_history %}
Consider the action history from the previous steps and the screenshot together, if actions from the previous steps don't yield positive impact, try to take an action on a different element.
Action history from previous steps:
```
{{ action_history }}
```{% endif %}
Current datetime, ISO format:
```
{{ local_datetime }}
```
