# This file was auto-generated by Fern from our API Definition.

import typing

SkyvernSchemasRunBlocksCredentialType = typing.Union[
    typing.Literal["skyvern", "bitwarden", "1password", "azure_vault"], typing.Any
]
