# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .act_action_data import ActActionData


class ActAction(UniversalBaseModel):
    """
    AI act action parameters.
    """

    intention: typing.Optional[str] = pydantic.Field(default=None)
    """
    Natural language prompt for the action
    """

    data: typing.Optional[ActActionData] = pydantic.Field(default=None)
    """
    Additional context data
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
