import jobApplicationImage from "@/assets/job-application.png";
import contactFormsImage from "@/assets/contact-forms.png";
import purchasingImage from "@/assets/purchasing.png";
import entityLookupImage from "@/assets/entity-lookup.png";
import invoiceImage from "@/assets/invoice-downloading.png";

// New WPID-named images
import invoiceDownloadingTemplateImage from "@/assets/wpid_462707285715519276.png";
import ss4TemplateImage from "@/assets/wpid_459004674259025010.png";
import delawareEntityTemplateImage from "@/assets/wpid_459013964273286908.png";
import becbParamTemplateImage from "@/assets/wpid_459004536820071490.png";
import samTemplateImage from "@/assets/wpid_459004755863403666.png";
import oigTemplateImage from "@/assets/wpid_459004731430231544.png";
import ashaTemplateImage from "@/assets/wpid_459004495289915148.png";
import bacbTemplateImage from "@/assets/wpid_459004516764751662.png";
import einTemplateImage from "@/assets/wpid_459004551041605070.png";
import irsEinTemplateImage from "@/assets/wpid_459009284178448812.png";

export const TEMPORARY_TEMPLATE_IMAGES: Record<string, string> = {
  wpid_353862309074493424: jobApplicationImage,
  wpid_351487857063054716: jobApplicationImage,
  wpid_356554024867713766: contactFormsImage,
  wpid_357290139535158898: purchasingImage,
  wpid_357290367168425660: entityLookupImage,
  wpid_369161749055928862: invoiceImage,
  wpid_369441201410076446: invoiceImage,
  wpid_462707285715519276: invoiceDownloadingTemplateImage,
  wpid_459004674259025010: ss4TemplateImage,
  wpid_459013964273286908: delawareEntityTemplateImage,
  wpid_459004536820071490: becbParamTemplateImage,
  wpid_459004755863403666: samTemplateImage,
  wpid_459004731430231544: oigTemplateImage,
  wpid_459004495289915148: ashaTemplateImage,
  wpid_459004516764751662: bacbTemplateImage,
  wpid_459004551041605070: einTemplateImage,
  wpid_459009284178448812: irsEinTemplateImage,
};
