"""add ai_fallback field to workflows table

Revision ID: d3ec63728c2a
Revises: 1bba8a38ddc7
Create Date: 2025-08-28 21:12:54.750395+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d3ec63728c2a"
down_revision: Union[str, None] = "1bba8a38ddc7"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflows", sa.Column("ai_fallback", sa.Boolean(), nullable=False, server_default=sa.false()))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "ai_fallback")
    # ### end Alembic commands ###
