.browser-stream {
  position: relative;
  width: 100%;
  height: 100%;
  min-height: 0;
  overflow: visible;

  transition: padding 0.2s ease-in-out;
}

.browser-stream.user-is-controlling {
  padding: 0rem;
}

.browser-stream .overlay {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  cursor: not-allowed;
}

.browser-stream.user-is-controlling .overlay {
  pointer-events: none;
  cursor: unset;
}

.browser-stream .take-control {
  transform: translateY(0);
  transition:
    transform 0.2s ease-in-out,
    opacity 0.2s ease-in-out;
  opacity: 0.3;
}

.browser-stream .take-control:not(.hide):hover {
  opacity: 1;
}

.browser-stream .take-control.hide {
  transform: translateY(100%);
  opacity: 0;
  pointer-events: none;
}

.browser-stream .relinquish-control {
  transform: translateY(0);
  transition:
    transform 0.2s ease-in-out,
    opacity 0.2s ease-in-out;
  opacity: 0.3;
  pointer-events: all;
}

.browser-stream .relinquish-control:not(.hide):hover {
  opacity: 1;
}

.browser-stream .relinquish-control.hide {
  transform: translateY(100%);
  opacity: 0;
  pointer-events: none;
}

.browser-stream > div > canvas {
  opacity: 0;
  animation: skyvern-anim-fadeIn 1s ease-in forwards;
}

.browser-stream > div {
  background: transparent !important;
  aspect-ratio: 16/9 !important;
}

.browser-stream .control-button {
  transition: 0.3s all ease-in-out;
  transform: translateY(0px);
}

.browser-stream .control-button.hide {
  opacity: 0;
  pointer-events: none;
  transform: translateY(15px);
}

@keyframes skyvern-anim-fadeIn {
  from {
    opacity: 0;
  }
  to {
    opacity: 1;
  }
}
