"""add run_signature to script block

Revision ID: b80c42316c94
Revises: 774e10939484
Create Date: 2025-10-14 22:32:57.466000+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "b80c42316c94"
down_revision: Union[str, None] = "774e10939484"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("script_blocks", sa.Column("run_signature", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("script_blocks", "run_signature")
    # ### end Alembic commands ###
