"""workflow copilot chat proposed workflow

Revision ID: 43217e31df12
Revises: ce791d022652
Create Date: 2026-01-27 20:20:36.952383+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "43217e31df12"
down_revision: Union[str, None] = "ce791d022652"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_copilot_chats", sa.Column("proposed_workflow", sa.JSON(), nullable=True))
    op.add_column("workflow_copilot_chats", sa.Column("auto_accept", sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_copilot_chats", "auto_accept")
    op.drop_column("workflow_copilot_chats", "proposed_workflow")
    # ### end Alembic commands ###
