"""oss sync alembic

Revision ID: 89d531e8b4ed
Revises: 2c34dee3304e
Create Date: 2025-11-06 08:05:01.832765+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "89d531e8b4ed"
down_revision: Union[str, None] = "2c34dee3304e"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("browser_profile_id", sa.String(), nullable=True))
    op.create_index(op.f("ix_workflow_runs_browser_profile_id"), "workflow_runs", ["browser_profile_id"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_workflow_runs_browser_profile_id"), table_name="workflow_runs")
    op.drop_column("workflow_runs", "browser_profile_id")
    # ### end Alembic commands ###
