# This file was auto-generated by Fern from our API Definition.

import enum


class SkyvernEnvironment(enum.Enum):
    CLOUD = "https://api.skyvern.com"
    STAGING = "https://api-staging.skyvern.com"
    LOCAL = "http://localhost:8000"
