"""script_blocks.script_file_id from required to optional

Revision ID: 20e960cf015d
Revises: 944ef972e5a8
Create Date: 2025-08-11 05:53:41.480218+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "20e960cf015d"
down_revision: Union[str, None] = "944ef972e5a8"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("script_blocks", "script_file_id", existing_type=sa.VARCHAR(), nullable=True)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("script_blocks", "script_file_id", existing_type=sa.VARCHAR(), nullable=False)
    # ### end Alembic commands ###
