"""add index for artifacts table (organization_id, created_at)

Revision ID: afeed80576cb
Revises: 2d2f165170f1
Create Date: 2025-06-19 22:59:31.436342+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "afeed80576cb"
down_revision: Union[str, None] = "2d2f165170f1"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index("artifacts_org_created_at_index", "artifacts", ["organization_id", "created_at"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index("artifacts_org_created_at_index", table_name="artifacts")
    # ### end Alembic commands ###
