"""add index

Revision ID: a720c991f779
Revises: 18a4ed894511
Create Date: 2026-01-15 07:13:16.022412+00:00

"""

from typing import Sequence, Union

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "a720c991f779"
down_revision: Union[str, None] = "18a4ed894511"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        op.f("ix_debug_sessions_browser_session_id"), "debug_sessions", ["browser_session_id"], unique=False
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_debug_sessions_browser_session_id"), table_name="debug_sessions")
    # ### end Alembic commands ###
