"""Add status column to debug_sessions table

Revision ID: 0ecb03206fc6
Revises: bddb54c1509b
Create Date: 2025-07-31 13:42:15.204085+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "0ecb03206fc6"
down_revision: Union[str, None] = "bddb54c1509b"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("debug_sessions", sa.Column("status", sa.String(), nullable=False, server_default="created"))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("debug_sessions", "status")
    # ### end Alembic commands ###
