"""add_sequential_key

Revision ID: 4925c34e8d58
Revises: 6692fb5a3e91
Create Date: 2025-09-24 03:47:37.324185+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "4925c34e8d58"
down_revision: Union[str, None] = "6692fb5a3e91"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("sequential_key", sa.String(), nullable=True))
    op.add_column("workflows", sa.Column("sequential_key", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflows", "sequential_key")
    op.drop_column("workflow_runs", "sequential_key")
    # ### end Alembic commands ###
