"""db migration created_by steps

Revision ID: 152354699b93
Revises: 44e95cb21d98
Create Date: 2025-12-04 21:20:20.166647+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "152354699b93"
down_revision: Union[str, None] = "44e95cb21d98"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("steps", sa.Column("created_by", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("steps", "created_by")
    # ### end Alembic commands ###
