"""add_cdp_address_in_task_and_workflow

Revision ID: afdfd14ac46d
Revises: 20e960cf015d
Create Date: 2025-08-21 03:13:24.973540+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "afdfd14ac46d"
down_revision: Union[str, None] = "20e960cf015d"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("observer_cruises", sa.Column("browser_address", sa.String(), nullable=True))
    op.add_column("tasks", sa.Column("browser_address", sa.String(), nullable=True))
    op.add_column("workflow_runs", sa.Column("browser_address", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "browser_address")
    op.drop_column("tasks", "browser_address")
    op.drop_column("observer_cruises", "browser_address")
    # ### end Alembic commands ###
