"""add browser type to pbs

Revision ID: 18a4ed894511
Revises: 3545c24f02f6
Create Date: 2026-01-07 07:35:47.209857+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "18a4ed894511"
down_revision: Union[str, None] = "3545c24f02f6"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("persistent_browser_sessions", sa.Column("browser_type", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("persistent_browser_sessions", "browser_type")
    # ### end Alembic commands ###
