"""oss sync alembic

Revision ID: 7581811d57b1
Revises: 89d531e8b4ed
Create Date: 2025-11-06 08:57:29.949001+00:00

"""

from typing import Sequence, Union

# revision identifiers, used by Alembic.
revision: str = "7581811d57b1"
down_revision: Union[str, None] = "89d531e8b4ed"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
