"""add download_timeout to tasks table

Revision ID: 1ab477ef80e4
Revises: cce87185dbb5
Create Date: 2025-10-06 15:04:12.103789+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "1ab477ef80e4"
down_revision: Union[str, None] = "cce87185dbb5"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("tasks", sa.Column("download_timeout", sa.Numeric(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("tasks", "download_timeout")
    # ### end Alembic commands ###
