"""add finished_at to steps table

Revision ID: d67d7c00ab02
Revises: 0135ee8b36b0
Create Date: 2025-08-07 15:30:06.604933+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d67d7c00ab02"
down_revision: Union[str, None] = "0135ee8b36b0"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("steps", sa.Column("finished_at", sa.DateTime(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("steps", "finished_at")
    # ### end Alembic commands ###
