@keyframes pulse-dash {
  0% {
    stroke-dasharray: 141.4 141.4;
    stroke-width: 6;
  }
  50% {
    stroke-dasharray: 10 11.4;
    stroke-width: 8;
  }
  100% {
    stroke-dasharray: 141.4 141.4;
    stroke-width: 6;
  }
}

@keyframes pulse-dash-small {
  0% {
    stroke-dasharray: 100.4 100.4;
    stroke-width: 3;
  }
  50% {
    stroke-dasharray: 10 11.4;
    stroke-width: 5;
  }
  100% {
    stroke-dasharray: 100.4 100.4;
    stroke-width: 3;
  }
}
