"""add ai_fallback to workflow_runs

Revision ID: d36daac4941e
Revises: c50ee6f26432
Create Date: 2025-10-01 21:07:38.185580+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "d36daac4941e"
down_revision: Union[str, None] = "c50ee6f26432"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("ai_fallback", sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "ai_fallback")
    # ### end Alembic commands ###
