# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .workflow_definition_yaml_blocks_item import WorkflowDefinitionYamlBlocksItem
from .workflow_definition_yaml_parameters_item import WorkflowDefinitionYamlParametersItem


class WorkflowDefinitionYaml(UniversalBaseModel):
    version: typing.Optional[int] = None
    parameters: typing.List[WorkflowDefinitionYamlParametersItem]
    blocks: typing.List[WorkflowDefinitionYamlBlocksItem]
    finally_block_label: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .for_loop_block_yaml import ForLoopBlockYaml  # noqa: E402, F401, I001

update_forward_refs(WorkflowDefinitionYaml)
