"""add_depends_on_workflow_run_id

Revision ID: 81351201bc8d
Revises: 1ab477ef80e4
Create Date: 2025-10-08 07:53:33.714723+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "81351201bc8d"
down_revision: Union[str, None] = "1ab477ef80e4"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("workflow_runs", sa.Column("depends_on_workflow_run_id", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("workflow_runs", "depends_on_workflow_run_id")
    # ### end Alembic commands ###
